unit F_BrushPenFont;

(*
Notes on this program...
- the UserInterface does not use any of Delphi's UI sophistication, eg actions,
  so that you can concentrate on the effects of the code rather than on what the UI is doing
- the fonts on the panels for the UI are set to fsBold, purely for legibility when projected at BUG
- not all properties/methods/values are shown here, but they usually follow the same principles
- anything in brackets is not part of TCanvas, but a specially-written utility routine
*)


interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  StdCtrls, ExtCtrls, Spin, ComCtrls, ColorGrd;


type
  TfrmBrushPenFont = class(TForm)
    pnlHeader: TPanel;
    pnlPen: TPanel;
    pnlBrush: TPanel;
    pnlFont: TPanel;
    boxBrush: TGroupBox;
    pntBrushExample: TPaintBox;
    boxBrushColour: TGroupBox;
    cgrBrushColour: TColorGrid;
    rdoBrushStyle: TRadioGroup;
    boxPen: TGroupBox;
    pntPenExample: TPaintBox;
    boxPenColour: TGroupBox;
    cgrPenColour: TColorGrid;
    rdoPenStyle: TRadioGroup;
    boxFont: TGroupBox;
    txtFontExample: TLabel;
    boxFontColour: TGroupBox;
    cgrFontColour: TColorGrid;
    boxFontStyle: TGroupBox;
    ckbFontBold: TCheckBox;
    ckbFontItalic: TCheckBox;
    ckbFontUnderline: TCheckBox;
    ckbFontStrikeout: TCheckBox;
    pnlOnScreen: TPanel;
    pntOnScreen: TPaintBox;
    ckbFontTransparent: TCheckBox;
    rdoTextAlignment: TRadioGroup;
    btnClear: TButton;
    boxGraduClears: TGroupBox;
    btnGraduVertical: TButton;
    btnGraduHorizontal: TButton;
    btnGraduPyramid: TButton;
    boxFontNameSize: TGroupBox;
    txtFontName: TLabel;
    spinFontSize: TSpinEdit;
    boxPenWidth: TGroupBox;
    spinPenWidth: TSpinEdit;
    ckbBrushUseBitmap: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure pntPenExamplePaint(Sender: TObject);
    procedure pntBrushExamplePaint(Sender: TObject);
    procedure cgrBrushColourMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure txtFontNameDblClick(Sender: TObject);
    procedure ckbFontTransparentClick(Sender: TObject);
    procedure cgrFontColourChange(Sender: TObject);
    procedure rdoTextAlignmentClick(Sender: TObject);
    procedure spinFontSizeChange(Sender: TObject);
    procedure ckbFontBoldClick(Sender: TObject);
    procedure pntOnScreenPaint(Sender: TObject);
    procedure btnClearClick(Sender: TObject);
    procedure btnGraduVerticalClick(Sender: TObject);
    procedure btnGraduHorizontalClick(Sender: TObject);
    procedure btnGraduPyramidClick(Sender: TObject);
    procedure SetupTheBrush(Sender: TObject);
    procedure SetupThePen(Sender: TObject);
    procedure ckbBrushUseBitmapClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmBrushPenFont: TfrmBrushPenFont;

implementation

uses
  Dialogs, // for TFontDialog
  ClipBrd, // for access to the Windows clipboard
  JPEG, // if we want to save to JPG format
  BugColourControl,
  BugCommon;

{$R *.DFM}

{.$.R bug.res}
// remove the dots if you want to load the bug logo (for the header panel) from this bound-in resource

// an example of a 'cracker' class, which simple redeclares a property as public instead of private
type PanelCanvas = class(TPanel)
     public
       property Canvas;
     end;

var
  bmpLogo: TBitmap;
  MarqueeRect: TRect;
  LinePoints: array of TPoint;
  bmpUndo,            // this could be a TImageList, which would allow multiple levels of undo
  bmpOffscreen,
  bmpBrush,
  bmpPaste: TBitmap;  // only assigned if actually pasting something


procedure TfrmBrushPenFont.FormCreate(Sender: TObject);
begin
  // load the example brush bitmap into an in-memory bitmap
  bmpBrush := TBitmap.Create;
  bmpBrush.LoadFromFile('Brush_8x8.bmp');

  // set up an in-memory bitmap to store the work so far, in case we want to undo something!
  bmpUndo:= TBitmap.Create; // no need to set the Size, that will be done by the Assign when bmpUndo is used

  // set up an in-memory bitmap, which will be the REAL work area, but held off-screen
  bmpOffscreen:= TBitmap.Create;
  bmpOffscreen.Width := pntOnScreen.Width;
  bmpOffscreen.Height := pntOnScreen.Height;

  // ensure that the Font controls initialise to the actual Font settings in use
  ckbFontBold.Checked := fsBold in bmpOffScreen.Canvas.Font.Style;
  ckbFontItalic.Checked := fsItalic in bmpOffScreen.Canvas.Font.Style;
  ckbFontUnderline.Checked := fsUnderline in bmpOffScreen.Canvas.Font.Style;
  ckbFontStrikeout.Checked := fsStrikeout in bmpOffScreen.Canvas.Font.Style;
  txtFontName.Caption := bmpOffScreen.Canvas.Font.Name;
  txtFontName.Transparent := ckbFontTransparent.Checked;
  spinFontSize.Value := bmpOffScreen.Canvas.Font.Size;
  txtFontExample.Font.Assign(bmpOffscreen.Canvas.Font);
end;

procedure TfrmBrushPenFont.FormDestroy(Sender: TObject);
begin
  bmpLogo.Free;
  bmpUndo.Free;
  bmpOffscreen.Canvas.Brush.Bitmap := nil;
  bmpOffscreen.Free;
  bmpBrush.free;
  LinePoints:= nil;  // the equivalent of free for a dynamic array
end;

procedure TfrmBrushPenFont.SetupThePen(Sender: TObject);
// NOTE Windows limits the effectiveness of Pen.Style v Pen.Width
// Width has to be 1 for any non-psSolid Style to be effective
begin
  with bmpOffscreen.Canvas.Pen
  do begin
       // uncomment this if you want to see the UI representation of the Note above
//       if (Sender = rdoPenStyle) and (rdoPenStyle.ItemIndex < 4)
//       then spinPenWidth.Value := 1
//       else if (Sender = spinPenWidth) and (spinPenWidth.Value > 1)
//            then rdoPenStyle.ItemIndex := 4;
       Color := cgrPenColour.ForegroundColor;
       case rdoPenStyle.ItemIndex of
         0: Style := psClear;
         1: Style := psDot;
         2: Style := psDash;
         3: Style := psDashDot;
         4: Style := psSolid;
       end; //case
       Width := spinPenWidth.Value;
     end;
  pntPenExample.Canvas.Pen.Assign(bmpOffscreen.Canvas.Pen);
  pntPenExamplePaint(Sender);
end;

procedure TfrmBrushPenFont.pntPenExamplePaint(Sender: TObject);
begin
  ClearRectangle(pntPenExample.Canvas, pntPenExample.ClientRect, boxPen.Color);
  pntPenExample.Canvas.Polyline([Point(0, pntPenExample.Height div 2),
                                 Point(pntPenExample.Width, pntPenExample.Height div 2)]);
end;

procedure TfrmBrushPenFont.SetupTheBrush(Sender: TObject);
// WARNING- Delphi tries to be clever...
// if you change the Style to bsClear, Delphi changes the Color to none, ie white
// if you change the Color when the Style is bsClear, Delphi changes the Style to bsSolid
// These combine to mean that you can't have a bsClear Brush with any colour to it!
begin
  // change the Style first...
  with bmpOffscreen.Canvas.Brush
  do case rdoBrushStyle.ItemIndex of
         0: Style := bsClear;
         1: Style := bsHorizontal;
         2: Style := bsVertical;
         3: Style := bsBDiagonal;
         4: Style := bsCross;
         5: Style := bsDiagCross;
         6: Style := bsSolid;
       end; //case
  //... then only change the colour if it's not going to change the Style back again!
  if rdoBrushStyle.ItemIndex <> 0 {bsClear}
  then bmpOffscreen.Canvas.Brush.Color := cgrBrushColour.BackgroundColor;

  pntBrushExample.Canvas.Brush.Assign(bmpOffscreen.Canvas.Brush);
  pntBrushExamplePaint(Sender);

  // and do the ancillary things affected by a change in the Brush colour
  txtFontExample.Color := cgrBrushColour.BackgroundColor;
  if Assigned(bmpPaste) and bmpPaste.Transparent
  then bmpPaste.TransparentColor := cgrBrushColour.BackgroundColor;
end;

procedure TfrmBrushPenFont.ckbBrushUseBitmapClick(Sender: TObject);
begin
  if ckbBrushUseBitmap.checked
  then begin
         bmpOffscreen.Canvas.Brush.Bitmap := bmpBrush;
         pntBrushExample.Canvas.Brush.Assign(bmpOffscreen.Canvas.Brush);
         pntBrushExamplePaint(Sender);
       end
  else begin
         bmpOffscreen.Canvas.Brush.Bitmap := nil;
         SetupTheBrush(Sender);
       end;
end; //procedure TfrmMain.ckbBrushUseBitmapClick

procedure TfrmBrushPenFont.pntBrushExamplePaint(Sender: TObject);
begin
  ClearRectangle(pntBrushExample.Canvas, pntBrushExample.ClientRect, boxBrush.Color);
  pntBrushExample.Canvas.Pen.Style := psClear; // avoids the border caused by the following Rectangle
  pntBrushExample.Canvas.Rectangle(pntBrushExample.ClientRect);
end;

procedure TfrmBrushPenFont.cgrBrushColourMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
// The Colorgrid uses Right-clicks to select backgroundcolour,
// but since this particular Colorgrid is only for backgroundcolor
// this is some code to allow Left-click to do it here (as well)
begin
  if mbLeft = Button
  then cgrBrushColour.BackgroundIndex := Y div (cgrBrushColour.Height div 16);
end;

procedure TfrmBrushPenFont.txtFontNameDblClick(Sender: TObject);
begin
  with TFontDialog.Create(nil)  // dynamic creation of a component that's only gonna be used very short-term
  do try
       Font.Assign(bmpOffscreen.Canvas.Font);
       if Execute
       then begin
               bmpOffscreen.Canvas.Font.Assign(Font);
               txtFontExample.Font.Assign(bmpOffscreen.Canvas.Font);
               txtFontName.Caption := Font.name;
               spinFontSize.Value := Font.Size;
               ckbFontBold.Checked := (fsBold in Font.Style);
               ckbFontItalic.Checked := (fsItalic in Font.Style);
               ckbFontUnderline.Checked := (fsUnderline in Font.Style);
               ckbFontStrikeout.Checked := (fsStrikeout in Font.Style);
            end; //if Execute
     finally
       Free;
     end;
end;

procedure TfrmBrushPenFont.ckbFontTransparentClick(Sender: TObject);
begin
  txtFontExample.Transparent := ckbFontTransparent.Checked;
end;

procedure TfrmBrushPenFont.cgrFontColourChange(Sender: TObject);
begin
  bmpOffscreen.Canvas.Font.Color := cgrFontColour.ForegroundColor;
  txtFontExample.Font.Assign(bmpOffscreen.Canvas.Font);
end;

procedure TfrmBrushPenFont.rdoTextAlignmentClick(Sender: TObject);
begin
  case rdoTextAlignment.ItemIndex of
  0: txtFontExample.Alignment := taLeftJustify;
  1: txtFontExample.Alignment := taCenter;
  2: txtFontExample.Alignment := taRightJustify;
  end;
end;

procedure TfrmBrushPenFont.spinFontSizeChange(Sender: TObject);
begin
  bmpOffscreen.Canvas.Font.Size := spinFontSize.Value;
  txtFontExample.Font.Assign(bmpOffscreen.Canvas.Font);
end;

procedure TfrmBrushPenFont.ckbFontBoldClick(Sender: TObject);
var
  TheStyle: TFontStyles;
begin
  TheStyle:= [];
  if ckbFontBold.Checked then TheStyle := TheStyle + [fsBold];
  if ckbFontItalic.Checked then TheStyle := TheStyle + [fsItalic];
  if ckbFontUnderline.Checked then TheStyle := TheStyle + [fsUnderline];
  if ckbFontStrikeout.Checked then TheStyle := TheStyle + [fsStrikeout];
  bmpOffscreen.Canvas.Font.Style := TheStyle;
  txtFontExample.Font.Assign(bmpOffscreen.Canvas.Font);
end;

procedure TfrmBrushPenFont.pntOnScreenPaint(Sender: TObject);
// this is the Paint routine called by Windows any time it needs to redraw pntOnScreen
begin
  pntOnScreen.Canvas.Draw(0, 0, bmpOffScreen);
end;

procedure TfrmBrushPenFont.btnClearClick(Sender: TObject);
begin
  ClearRectangle(bmpOffscreen.Canvas, pntOnscreen.ClientRect, cgrBrushColour.BackgroundColor);
  pntOnScreen.Repaint;
  if not IsRectEmpty(MarqueeRect)   // Windows API, but simple to port to Kylix
  then MarqueeRect := Rect(0,0,0,0);
end;

procedure TfrmBrushPenFont.btnGraduVerticalClick(Sender: TObject);
begin
  GraduClearRectangle(bmpOffscreen.Canvas, pntOnscreen.ClientRect, cgrBrushColour.BackgroundColor, cgrPenColour.ForegroundColor);
  pntOnScreen.Repaint;
  if not IsRectEmpty(MarqueeRect)   // Windows API, but simple to port to Kylix
  then MarqueeRect := Rect(0,0,0,0);
end;

procedure TfrmBrushPenFont.btnGraduHorizontalClick(Sender: TObject);
begin
  GraduClearRectangle(bmpOffscreen.Canvas, pntOnscreen.ClientRect, cgrBrushColour.BackgroundColor, cgrPenColour.ForegroundColor, True);
  pntOnScreen.Repaint;
  if not IsRectEmpty(MarqueeRect)   // Windows API, but simple to port to Kylix
  then MarqueeRect := Rect(0,0,0,0);
end;

procedure TfrmBrushPenFont.btnGraduPyramidClick(Sender: TObject);
begin
  PyramidClearRectangle(bmpOffscreen.Canvas, pntOnscreen.ClientRect, cgrBrushColour.BackgroundColor, cgrPenColour.ForegroundColor);
  pntOnScreen.Repaint;
  if not IsRectEmpty(MarqueeRect)   // Windows API, but simple to port to Kylix
  then MarqueeRect := Rect(0,0,0,0);
end;

end.


